
# q-expansion of newform 7504.2.a.be, downloaded from the LMFDB on 16 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7504
weight = 2
poly_data = [-1, -3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-2, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0], [1, 1, 0], [1, 0, 1], [-1, 0, 0], [2, -2, 3], [-1, 1, -2], [-3, 1, 1], [1, 3, 0], [4, 2, 0], [-4, -4, 1], [1, -4, 2], [-1, -3, 3], [-1, 2, 1], [1, 6, -3], [0, -2, -5], [2, 3, -1], [10, 0, 1], [2, 3, -3], [-1, 0, 0], [13, 1, -1], [-1, -4, -1], [4, -2, -2], [-10, 2, -2], [10, -3, 7], [5, -3, 0], [-5, -4, 6], [-2, -11, 10], [-4, 0, -7], [2, 4, 1], [-1, 2, -5], [-5, 0, 3], [5, -3, 2], [-4, 6, -13], [10, -6, 9], [-5, 3, -2], [-2, 10, -5], [8, -3, 0], [-2, -8, 1], [1, -2, -1], [-1, -5, 3], [16, 3, -2], [14, 4, 1], [1, 5, -9], [-1, 6, 9], [-1, -13, 7], [10, -5, 4], [1, -15, 3], [-2, 10, -17], [2, 4, 0], [-7, 10, -11], [6, -1, -1], [7, 7, -7], [2, -12, 15], [-4, 8, -2], [12, -4, -4], [-11, 0, -2], [-1, -6, -10], [-2, 3, 6], [-13, 9, -6], [10, 1, 2], [-11, -12, 9], [9, -6, -6], [13, -1, -7], [4, 0, 19], [23, 0, 0], [-4, -5, 6], [-11, -15, 9], [8, 6, -9], [14, -8, -6], [-1, 2, 5], [-9, -2, -11], [-22, -3, 5], [4, -9, 15], [-1, 12, -3], [19, 12, -9], [-2, -4, 6], [-12, -8, 1], [8, -15, -12], [7, -3, -14], [14, -6, 12], [8, 8, -2], [11, -14, 19], [-4, -2, 0], [-4, 12, 3], [13, -9, 15], [30, -2, 1], [5, 8, -11], [11, -4, -10], [-21, -15, 12], [4, -3, 15], [7, 26, -15], [-12, 5, -19], [10, -9, 12], [-9, -6, -6], [19, -12, 9], [-11, -20, 17], [4, 9, -20], [-10, 3, -19], [-11, -9, 0], [-13, -3, -6], [10, 6, -15], [4, 2, -21], [-2, 18, -14], [-15, -4, -1], [19, -9, 18], [-13, -6, 18], [-1, -22, 7], [8, 16, -6], [11, 19, 0], [-13, -18, 9], [4, 4, -20], [-13, 8, -4], [-3, -3, 15], [-26, 3, -15], [19, -1, 8], [-11, -22, 12], [16, 1, 1], [0, -19, 5], [-4, 1, 12], [-8, -29, 11], [2, -23, 7], [-19, -1, -13], [-24, 14, -4], [12, -10, 29], [10, 22, -26], [-1, -13, -8], [5, -2, 4], [25, -13, 9], [7, -9, -6], [11, 23, -22], [-2, 30, -6], [-7, 11, -5], [-23, 9, -3], [11, -6, -3], [10, 18, -20], [8, 1, 16], [7, -13, 27], [-2, 3, -3], [8, -22, 31], [17, 3, 17], [4, -12, -15], [5, 31, -12], [7, 10, -26], [-13, -16, 25], [-7, 3, -3], [22, 19, -13], [-19, 6, -3], [8, 6, -13], [-2, -5, -20], [14, 9, 2], [-19, -9, -15], [-8, -8, -13], [10, -15, 6], [-5, -7, 6], [-8, -6, -15], [15, -15, 30], [-29, 9, -12], [-11, 21, 4], [5, -1, -4], [-25, -12, -4], [23, -6, 20], [6, 9, -15], [10, -15, 3], [21, -26, -2], [27, -8, 28], [-5, 21, -20], [7, -6, 24], [-13, -18, 15], [5, -17, 16], [16, 0, -11], [22, 9, -14], [35, 6, 9], [-33, -5, -14], [-19, -4, 20], [-32, 11, 2], [10, -14, 17], [-20, 5, -1], [10, -4, -12], [-29, 1, 1], [26, 10, -2], [-11, 24, 18], [-22, 11, 10], [5, 18, -15], [13, -30, 4], [-1, 10, -27], [-18, 19, 1], [2, 13, -26], [-14, -11, 4], [17, -18, -3], [11, 16, -21], [-19, 4, 7], [15, -15, 3], [-14, -11, 1], [-8, 6, 16], [3, 14, 8], [-24, -2, 19], [11, 27, 0], [2, 11, 14], [-16, -10, 4], [-17, 14, 18], [-21, 7, 7], [10, -30, -3], [11, -6, 0], [-7, 21, -42], [30, 0, -9], [-11, 17, -9], [19, -16, -4], [10, -7, -21], [-11, 4, -7], [-14, 16, -5], [-31, 21, -12], [-14, 31, -4], [-5, 36, -12], [21, -23, 13], [11, -8, -3], [-16, -24, 18], [-2, 12, -30], [1, -28, 27], [-4, 12, -16], [-22, 26, -14], [14, -18, 45], [-23, -15, -6], [-37, 4, 6], [28, -9, 6], [13, 1, -7], [-55, 11, -4], [27, 5, -7], [-40, -11, 18], [-5, 21, 12], [40, -7, 21], [-25, 25, -5], [10, 20, 8], [1, -17, 4], [19, 14, -30], [19, 21, -15], [-30, -4, 8], [23, 18, -24], [-31, -14, 9], [6, 15, 6], [-25, 8, -8], [18, -4, 11], [25, 45, -33], [-35, -15, 0], [-1, 22, -38], [-16, -40, 16], [7, 17, -21], [22, 9, -9], [36, 29, -22], [-29, -1, 11], [-9, 15, -42], [53, -6, 0], [-33, 17, 11], [-47, -6, -14], [-25, 26, -13], [17, 27, -13], [9, -32, 10], [-10, -4, -16], [-32, -11, -8], [29, 29, -14], [8, 3, -12], [-44, 18, 3], [33, 18, 0], [14, -25, -4], [20, -30, -15], [26, 27, -22], [16, -21, -21], [-7, -4, -23], [-1, 5, -38], [1, -42, 0], [-16, 3, -1], [26, 9, -12], [-8, 3, -3], [17, -21, 6], [13, -7, 2], [-13, 19, -14], [25, 4, 10], [-4, 9, 17], [-7, -8, -23], [-16, -6, 30], [28, -13, 0], [16, -3, 22], [-50, 12, -12], [22, 10, 5], [41, 11, -19], [4, 0, -36], [26, 24, -22], [14, -15, 42], [33, 3, 18], [-38, 18, -27], [25, 4, 26], [14, 9, -10], [23, -23, 27], [-52, -5, 9], [-41, -23, 26], [20, -12, 6], [-39, 19, -8], [-11, -45, 18], [4, -29, -7], [33, -5, 22], [-14, 17, 17], [26, -11, -17], [-19, -5, 15], [-17, -24, 36], [-47, 16, -22], [-29, 17, -22], [-52, 13, 16], [-27, 35, -4], [-13, -11, 19], [9, -38, 31], [-28, -14, 34], [35, 33, -25], [14, -21, -31], [12, -7, -7], [4, -9, 0], [17, 8, 28], [-19, 0, 9], [-3, 0, -6], [-43, 13, 9], [-52, -20, 22], [-9, -6, -15], [34, 1, 28], [8, 11, -25], [63, 9, -6], [43, 37, -20], [-43, 21, -1], [5, 1, -14], [-2, 43, -14], [46, 20, -13], [-21, -11, -29], [-29, -23, 14], [-28, -42, 12], [-5, -16, 48], [-17, 24, 24], [-1, 43, -39], [7, 5, 17], [-8, 7, -31], [2, 23, 5], [13, -42, 4], [35, -4, 2], [7, 3, 12], [-67, -5, -2], [43, 10, -28], [8, 32, -20], [-26, -19, -4], [48, -22, 17], [26, 50, -44], [-34, -27, 36], [-29, -9, 18], [-22, -4, -26], [39, -19, -4], [-17, -32, 43], [-16, -32, 4], [26, 32, -17], [10, 39, -30], [2, 39, 0], [25, -29, 17], [7, 17, -15], [34, -9, 7], [13, 27, -32], [-14, -14, 32], [47, 11, 5], [12, 29, -1], [35, 1, 0], [30, -38, 25], [-17, 29, 14], [14, 34, -32], [52, -17, 20], [19, -3, 3], [65, -7, -4], [-5, -3, -8], [7, 34, -50], [23, -15, 17], [-21, 42, 9], [-26, -33, 6], [-16, -4, 5], [17, -5, -15], [-15, 21, 24], [-37, 15, 15], [48, -22, 11], [-37, 24, -3], [-10, -3, 44], [14, -28, -8], [1, -28, 26], [-57, -11, -11], [-26, 27, 9], [-12, 32, -4], [-41, -34, 8], [17, 36, -51], [64, -24, -6], [34, -11, 2], [20, -37, -4], [-31, -5, 0], [-3, 32, -43], [-23, -15, -3], [-27, 4, -29], [29, -39, -30], [52, -14, -23], [-17, -5, 14], [19, -30, -18], [30, -23, -29], [14, 13, 22], [-34, -2, 21], [37, -2, 25], [5, 25, -21], [38, -9, 5], [-17, 27, -45], [-1, -31, 23], [1, -14, -28], [-26, -31, -7], [-19, 9, -10], [56, -17, -5], [23, -14, 36], [-5, -11, -43], [10, -34, 2], [29, -19, 44], [21, 29, 11], [4, -52, 42], [37, 31, 7], [40, 1, -7], [-13, -48, 8], [27, 25, -29], [-10, -24, 36], [4, 43, -13], [3, -12, 6], [-4, 2, 29], [14, 33, -34], [20, 45, -19], [-40, 26, -5], [31, 46, -44], [12, -40, 44]]
